"use strict";
var alsatian_core_1 = require("../core/alsatian-core");
var CliTestRunner = (function () {
    function CliTestRunner(_testRunner) {
        this._testRunner = _testRunner;
        if (!_testRunner) {
            throw new TypeError("_testRunner must not be null or undefined.");
        }
    }
    CliTestRunner.prototype.run = function (testSet, timeout) {
        try {
            var testRunPromise = this._testRunner.run(testSet, timeout);
            testRunPromise.then(function (results) {
                if (results.outcome === alsatian_core_1.TestOutcome.Error || results.outcome === alsatian_core_1.TestOutcome.Fail) {
                    process.exit(1);
                }
                else {
                    process.exit(0);
                }
            })
                .catch(this._handleTestSetRunError);
        }
        catch (error) {
            this._handleTestSetRunError(error);
        }
    };
    CliTestRunner.prototype._handleTestSetRunError = function (error) {
        process.stderr.write(error.message + "\n");
        process.exit(1);
    };
    return CliTestRunner;
}());
exports.CliTestRunner = CliTestRunner;
//# sourceMappingURL=cli-test-runner.js.map