/**
 * Allows checking of test outcomes
 * @param actualValue - the value or function under test
 */
export declare function Expect(actualValue: any): Matcher;
/**
 * Gives functionality to ensure the outcome of a test is as expected
 */
export declare class Matcher {
    private _actualValue;
    private _shouldMatch;
    constructor(actualValue: any);
    /**
     * Any subsequent matcher function will be looking for the opposite criteria
     */
    not: Matcher;
    /**
     * Checks that a value is identical to another
     * @param expectedValue - the value that will be used to match
     */
    toBe(expectedValue: any): void;
    /**
     * Checks that a value is equal to another (for objects the function will check for deep equality)
     * @param expectedValue - the value that will be used to match
     */
    toEqual(expectedValue: any): void;
    private _checkObjectsAreDeepEqual(objectA, objectB);
    /**
     * Checks that a value conforms to a regular expression
     * @param regex - the regular expression that the actual value should match
     */
    toMatch(regex: RegExp): void;
    /**
     * Checks that a value is not undefined
     */
    toBeDefined(): void;
    /**
     * Checks that a value is null
     */
    toBeNull(): void;
    /**
     * Checks that a value is equivalent to boolean true
     */
    toBeTruthy(): void;
    /**
     * Checks that a string contains another string or an array contains a specific item
     * @param expectedContent - the string or array item that the value should contain
     */
    toContain(expectedContent: any): void;
    /**
     * Checks that a number is less than a given limit
     * @param upperLimit - the number that the number under test should be less than
     */
    toBeLessThan(upperLimit: number): void;
    /**
     * Checks that a number is greater than a given limit
     * @param lowerLimit - the number that the number under test should be greater than
     */
    toBeGreaterThan(lowerLimit: number): void;
    /**
     * Checks that a function throws an error when executed
     */
    toThrow(): void;
    /**
     * Checks that a function throws a specific error
     * @param errorType - the type of error that should be thrown
     * @param errorMessage - the message that the error should have
     */
    toThrowError(errorType: new (...args: Array<any>) => Error, errorMessage: string): void;
    /**
     * Checks that a spy has been called
     */
    toHaveBeenCalled(): void;
    /**
     * Checks that a spy has been called with the specified arguments
     * @param args - a list of arguments that the spy should have been called with
     */
    toHaveBeenCalledWith(...args: Array<any>): void;
    /**
     * Checks that a property spy has been set
     */
    toHaveBeenSet(): void;
    /**
     * Checks that a property spy has been set to a specific value
     * @param value - a value to which the property should be set to
     */
    toHaveBeenSetTo(value: any): void;
}
