"use strict";
var alsatian_core_1 = require("../alsatian-core");
var promise_1 = require("../../promise/promise");
var _errors_1 = require("../_errors");
var TestItem = (function () {
    function TestItem(_testFixture, _test, _testCase) {
        this._testFixture = _testFixture;
        this._test = _test;
        this._testCase = _testCase;
    }
    Object.defineProperty(TestItem.prototype, "testCase", {
        get: function () {
            return this._testCase;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TestItem.prototype, "test", {
        get: function () {
            return this._test;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TestItem.prototype, "testFixture", {
        get: function () {
            return this._testFixture;
        },
        enumerable: true,
        configurable: true
    });
    TestItem.prototype.run = function (timeout) {
        var _this = this;
        if (this._test.ignored) {
            return new promise_1.Promise(function (resolve, reject) {
                resolve({ test: _this._test });
            });
        }
        else {
            this._setup();
            if (this._test.isAsync) {
                return this._runAsync(timeout);
            }
            else {
                return this._runSync();
            }
        }
    };
    TestItem.prototype._runSync = function () {
        var _this = this;
        return new promise_1.Promise(function (resolve, reject) {
            try {
                _this._testFixture.fixture[_this._test.key].apply(_this._testFixture.fixture, _this._testCase.arguments);
                _this._reportResult(resolve);
            }
            catch (error) {
                _this._reportResult(resolve, error);
            }
        });
    };
    TestItem.prototype._runAsync = function (timeout) {
        var _this = this;
        return new promise_1.Promise(function (resolve, reject) {
            var timeoutExpired = false;
            var testPromise = _this._testFixture.fixture[_this._test.key].apply(_this._testFixture.fixture, _this._testCase.arguments);
            var timeoutCheck = null;
            testPromise.then(function () {
                if (!timeoutExpired) {
                    clearTimeout(timeoutCheck);
                    _this._reportResult(resolve);
                }
            })
                .catch(function (error) {
                console.log(error);
                clearTimeout(timeoutCheck);
                _this._reportResult(resolve, error);
            });
            var testTimeout = _this._test.timeout || timeout;
            timeoutCheck = setTimeout(function () {
                timeoutExpired = true;
                var error = new _errors_1.TestTimeoutError(testTimeout);
                _this._reportResult(resolve, error);
            }, testTimeout);
        });
    };
    TestItem.prototype._reportResult = function (resolve, error) {
        this._teardown();
        resolve({ test: this._test, error: error });
    };
    TestItem.prototype._setup = function () {
        var _this = this;
        var setupFunctions = Reflect.getMetadata(alsatian_core_1.METADATA_KEYS.SETUP, this._testFixture.fixture);
        if (setupFunctions) {
            setupFunctions.forEach(function (setupFunction) {
                _this._testFixture.fixture[setupFunction].call(_this._testFixture.fixture);
            });
        }
    };
    TestItem.prototype._teardown = function () {
        var _this = this;
        var teardownFunctions = Reflect.getMetadata(alsatian_core_1.METADATA_KEYS.TEARDOWN, this._testFixture.fixture);
        if (teardownFunctions) {
            teardownFunctions.forEach(function (teardownFunction) {
                _this._testFixture.fixture[teardownFunction].call(_this._testFixture.fixture);
            });
        }
    };
    return TestItem;
}());
exports.TestItem = TestItem;
//# sourceMappingURL=test-item.js.map